/**
 * @file
 * Contains sam_simplify.js.
 */
(function ($, Drupal) {

  "use strict";

  Drupal.behaviors.SamSimplifyBehavior = {
    attach: function (context, drupalSettings) {

      $('body', context).once('sam.simplify').each(function () {
        // If there is nothing to simplify on the page, just bail out.
        const $all_elements_to_simplify = $('.sam-wrapper-simplify tr.sam-simplify');
        if ($all_elements_to_simplify.length === 0) {
          return;
        }

        // Hide all rows that were marked as such.
        $all_elements_to_simplify.hide();

        // Append the button(s) that will be used to add other elements.
        const $wrappers = $('.sam-wrapper-simplify');
        const button_label = Drupal.t('Add another item');
        $wrappers.each(function (i) {
          const button_html = '<input class="button form-submit sam-add-more-button" value="' + button_label + '">';
          const $wrapper = $(this);
          // Add the button with some help text indicating how many spots are
          // left.
          let help_text = Drupal.formatPlural($wrapper.find('tr.sam-simplify').length, '@count additional item can be added', '@count additional items can be added');

          $wrapper.parent().append(button_html + '<span class="sam-add-more-help">' + help_text + '</span>');
        });

        // Attach the click behavior to the buttons.
        $('.sam-add-more-button').on('click', function (e) {
          const $this = $(this);
          const $wrapper = $this.parent().find('.sam-wrapper-simplify');
          let $rows = $wrapper.find('tr.sam-simplify');

          // Show one additional empty element for each click.
          $rows.first().removeClass('sam-simplify').show();

          // Update the help text.
          $rows = $wrapper.find('tr.sam-simplify');
          if ($rows.length > 0) {
            const help_text = Drupal.formatPlural($rows.length, '@count additional item can be added', '@count additional items can be added');
            $this.next('.sam-add-more-help').html(help_text);
          }
          // If this was the last one, hide the button and the help text.
          else if ($rows.length === 0) {
            $this.next('.sam-add-more-help').hide();
            $this.hide();
          }
        });
      });

    }
  };

})(jQuery, Drupal, drupalSettings);
